import java.awt.event.*;
import java.util.*;
import net.java.games.input.*;
import net.java.games.input.Component;
import net.java.games.input.Component.Identifier;

/*
 * 入力クラス
 * <pre>
 * 入力系コントローラを管理
 *
 * 利用する場合は、
 * （１）activePAD：ゲームパッドの状態を取得する設定
 * （２）poll：ポーリング（状態取得）
 * （３）isKeyPressed：方向キー／ボタン押下中かをどうかをチェック
 * を使用してください。
 * </pre>
 */
public class InputControl {

	/** ボタン一覧 */
	private static final Identifier.Button[]	btns				= { Identifier.Button._0, Identifier.Button._1, Identifier.Button._2, Identifier.Button._3, Identifier.Button._4,
																		Identifier.Button._5, Identifier.Button._6, Identifier.Button._7, Identifier.Button._8, Identifier.Button._9,
																		Identifier.Button._10, Identifier.Button._11, Identifier.Button._12, Identifier.Button._13, Identifier.Button._14,
																		Identifier.Button._15, Identifier.Button._16, Identifier.Button._17, Identifier.Button._18, Identifier.Button._19,
																		Identifier.Button._20, Identifier.Button._21, Identifier.Button._22, Identifier.Button._23, Identifier.Button._24,
																		Identifier.Button._25, Identifier.Button._26, Identifier.Button._27, Identifier.Button._28, Identifier.Button._29,
																		Identifier.Button._30, Identifier.Button._31
	};

	/** コントローラマップ */
	private static HashMap<String, Controller>	mapController		= new HashMap<String ,Controller>();

	/** 状態取得中のコントローラ */
	private static  Controller					controllerPoll		= null;

	//
	//
	//

	/**
	 * コントローラ一覧取得
	 */
	public static Controller[] getControllers(){

		// コントローラ一覧取得
        Controller[] controllers = ControllerEnvironment.getDefaultEnvironment().getControllers();

		// リターン
		return controllers;

    } // end getControllers

	/**
	 * コントローラ情報を取得
	 */
	public static void printController(){

		// コントローラ一覧取得
		Controller[] controllers = InputControl.getControllers();

		// ループ
		int i = 0;

		// コントローラ一覧の件数分、ループ
        for(Controller controller : controllers){

			// ループ表示
			System.out.println(i);

			// 各種情報を表示
			System.out.println("Name : " + controller.getName());
			System.out.println("Type : " + controller.getType());

			// ブランク行
			System.out.println("");

			// ループ＋１
			i++;

		} // end for

	} // end printController

	/**
	 * ゲームパッドの状態を取得する設定
	 * <pre>
	 * ゲームパッドからの状態を取得するようにします。
	 * パッドとして認識した１台目を取得できるようにします。
	 * </pre>
	 * @return Controller オブジェクト
	 */
	public static Controller activePAD(){

		return InputControl.activePAD(0);

	}

	/**
	 * ゲームパッドの状態を取得する設定
	 * <pre>
	 * ゲームパッドからの状態を取得するようにします。
	 * 複数パッドを取得できるようにします。
	 * </pre>
	 * @param index パッドを認識した順番 ０から指定ください（１台目は０）
	 * @return Controller オブジェクト
	 */
	public static Controller activePAD(int index){

		// コントローラ一覧取得
		Controller[] controllers = InputControl.getControllers();

		// 認識した数
		int i = 0;

		// コントローラ一覧の件数分、ループ
        for(Controller controller : controllers){

			// タイプを取得
			Controller.Type type = controller.getType();

			// パッドの場合、
			if(type == Controller.Type.GAMEPAD || type == Controller.Type.STICK || type == Controller.Type.WHEEL || type == Controller.Type.FINGERSTICK){

				// 認識した番号が合う場合、
				if(i == index){

					// コントローラマップに追加
					mapController.put(String.valueOf(i), controller);

					// コントローラをリターン
					return controller;

				} // end if 認識した番号が合う場合

			} // end if GAMEPAD または STICK の場合

		} // end for コントローラ一覧の件数分、ループ

		// null でリターン
		return null;

	} // activePAD

	/**
	 * ポーリング（状態取得）
	 * <pre>
	 * パッドの状態を取得します。
	 * パッドを認識した０番を利用します。
	 * <pre>
	 */
	public static boolean poll(){

		return InputControl.poll(0);

	}

	/**
	 * ポーリング（状態取得）
	 * <pre>
	 * パッドの状態を取得します。
	 * <pre>
	 * @param index パッドを認識した順番 ０から指定ください（１台目は０）
	 */
	public static boolean poll(int index){

		// コントローラマップからコントローラを取得
		Controller controller = mapController.get(String.valueOf(index));

		// コントローラが取得できない場合、
		if(controller == null){
			// スロー
			throw new NullPointerException("PAD(index) is NULL!! \n Please call activePAD.");
		}

		// クラス変数を置き換え
		controllerPoll = controller;

		// ポーリング（状態取得）しリターン
		return controller.poll();

	} // end poll

	/**
	 * 方向キー／ボタン押下中かをどうかをチェック
	 * <pre>
	 * 処理のしやすさを考慮し、java.awt.event.KeyEventに割り当てます。
	 * キーコードとして、
	 * 上：VK_UP(0x26－38)、下：VK_DOWN(0x28－40)、左：VK_LEFT(0x25－37)、右：VK_RIGHT(0x27－39)
	 * １ボタン：0 ～ 32ボタン：31
	 * として、割り当てます。
	 * 当メソッドを呼ぶ前にpollメソッドを呼び出してください。
	 * pollメソッド呼び出した時のパッドの状態を取得します。
	 * </pre>
	 * @param code キーコード。
	 * @return boolean true：押している ／ false：押していない キーコードが予期しないものは false
	 * @exception NullPointerException ゲームパッドの状態を取得する設定をしていない。
	 * @exception IllegalArgumentException キーコードが対応していない。
	 */
	public static boolean isKeyPressed(int code) throws NullPointerException {

		return InputControl.isKeyPressed(new int[]{code});

	} // end isKeyPressed

	/**
	 * 方向キー／ボタン押下中かをどうかをチェック（複数ボタン）
	 * <pre>
	 * 複数のボタンをチェックし、いずれかのボタンが押されていれば true をリターンします。
	 * 処理のしやすさを考慮し、java.awt.event.KeyEventに割り当てます。
	 * キーコードとして、
	 * 上：VK_UP(0x26－38)、下：VK_DOWN(0x28－40)、左：VK_LEFT(0x25－37)、右：VK_RIGHT(0x27－39)
	 * １ボタン：0 ～ 32ボタン：31
	 * として、割り当てます。
	 * 当メソッドを呼ぶ前にpollメソッドを呼び出してください。
	 * pollメソッド呼び出した時のパッドの状態を取得します。
	 * </pre>
	 * @param codes キーコード一覧。nullの場合は、処理を行わず、falseでリターンします。
	 * @return boolean true：押している ／ false：押していない キーコードが予期しないものは false
	 * @exception NullPointerException ゲームパッドの状態を取得する設定をしていない。
	 * @exception IllegalArgumentException キーコードが対応していない。
	 */
	public static boolean isKeyPressed(int[] codes) throws NullPointerException {

		// キーコード一覧がnullの場合、
		if(codes == null){
			// リターン
			return false;
		}

		// コントローラが取得できない場合、
		if(controllerPoll == null){
			// スロー
			throw new NullPointerException("controllerPoll is NULL!! \n Please call poll method.");
		}

		// X方向取得
		float x0 = controllerPoll.getComponent(Identifier.Axis.X).getPollData();

		// Y方向取得
		float y0 = controllerPoll.getComponent(Identifier.Axis.Y).getPollData();

		// キーコード一覧の件数を取得
		int countCode = codes.length;

		// キーコード一覧の件数分、ループ
		for(int i = 0; i < countCode; i++){

			// キーコード取得
			int code = codes[i];

			// 対応していないキーコードの場合、
			if(0 <= code && code <= 31 || KeyEvent.VK_LEFT <= code && code <= KeyEvent.VK_DOWN){

				// OK

			}else{
				// スロー
				throw new IllegalArgumentException("code is incorrect. code = " + code);
			}

			// キーコードによる判定
			switch(code){

				// 左
				case KeyEvent.VK_LEFT:

					// 押している場合、
					if(x0 < -0.1){
						// リターン
						return true;
					}

					break;

				// 右
				case KeyEvent.VK_RIGHT:

					// 押している場合、
					if(x0 > 0.1){
						// リターン
						return true;
					}

					break;

				// 上
				case KeyEvent.VK_UP:

					// 押している場合、
					if(y0 < -0.1){
						// リターン
						return true;
					}

					break;

				// 下
				case KeyEvent.VK_DOWN:

					// 押している場合、
					if(y0 > 0.1){
						// リターン
						return true;
					}

					break;

				// ボタン０～３１
				default:

					// ボタンの状態を取得
					Component component = controllerPoll.getComponent(btns[code]);

					// コンポーネントが取得できた場合、
					// （Buffaloの1204Pシリーズは、取得できないボタンあり）
					if(component != null){

						float b0 = component.getPollData();

						// 押している場合、
						if(b0 > 0.1){
							// リターン
							return true;
						}

					} // end if コンポーネントが取得できた場合

					break;

			} // end swith キーコードによる判定

		} // end for キーコード一覧の件数分、ループ

		// リターン
		return false;

	} // isKeyPressed

	//
	// ここは補足ですが、
	// activePAD：ゲームパッドの状態を取得する設定
	// について、なぜ状態を取得するパッドを決めなくてはいけないか、説明させていただきます。
	// 絞らなくても、できないことはないです。
	// ですが、リソース消費を考えてです。必要なもののみに絞っています。
	// なんでもかんでも必要以上に情報を取得することは、リソースの観点から望ましいものではありません。
	// 塵も積もれば山となる。です。
	// できるだけリソースを消費しないプログラムに心がけましょう。
	//

}
