import java.util.HashMap;

import javafx.collections.ObservableList;
import javafx.scene.Node;
import javafx.scene.Group;
import javafx.geometry.Bounds;
import javafx.scene.transform.Translate;
import javafx.scene.transform.Rotate;
import javafx.scene.shape.MeshView;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.paint.Color;

import javafx.event.EventType;
import javafx.event.EventHandler;

/**
 * 旧型戦艦 ゴル （ゴル級）
 * <pre>
 * 仕様をどんどん追加していくので、
 * プログラム的には、ひじょうに見にくいものとなってしまいました。
 * （旧型を改修した感じがでている？！こじつけ！）
 * あまりいいプログラムではないことを、ここでお伝えしておきます。
 * 反面教師とでも、思ってください。
 * 見やすい、読みやすいプログラムに心がけていきましょう♪
 * ここだけの話、砲台の回転がかなりめんどかったっす。はい。
 * </pre>
 */
public class PRSpriteBattleShip extends PRSpriteT {

	/** 耐久度 */
	private 	int						LIFE					= 200;

	/** 装甲の色 */
	private static final Color			COLOR_ARMOR				= Color.color(0.8, 0.3, 0.3);

	/** フェーズ */
	private		int						phase					= 0;

	/** 倍率 */
	private		double					scale					= 1.0;

	/** 座標 */
	private 	double					x						= 0.0;
	private 	double					y						= 0.0;
	private 	double					z						= 0.0;

	/** 時間 */
	private 	int						time					= 0;

	/** 角度 */
	private		int						angle					= 0;
	private		double[]				angleCannons			= {180, 180, 0};

	/** 発射制御 */
	private		int[]					phaseCannons			= {0, 0, 0};
	private		int[]					timeCannons				= {180, 240, 300};

	/** 耐久度 */
	private 	int						life					= LIFE;

	/** 移動方向 */
	private		int						way						= 1;

	/** 待ち時間 */
	private		int						timeWait				= 0;

	/** 敵の弾発射時間 */
	private		int						timeTamaTeki			= 240;

	/** 砲台 */
	private		Group[]					groupCannons			= null;

	/** マテリアル */
	private		PhongMaterial			materail				= null;

	/** 元の色 */
	private		Color					colorDefault			= null;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteBattleShip.class, "3d/BattleShip.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 */
	public PRSpriteBattleShip(PRApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// オブジェクト
			HashMap<String, Group> map = ObjReader.create(this);

			// 縮小する
			super.setScaleX(scale);
			super.setScaleY(scale);
			super.setScaleZ(scale);

			// 座標設定
			x = 0.0;
			y = -2.0;
			z = 300.0;

			// 速度設定
			mz = -2.0;

			// 砲台を保持
			groupCannons = new Group[3];
			groupCannons[0] = map.get("Cannon001");
			groupCannons[1] = map.get("Cannon002");
			groupCannons[2] = map.get("Cannon003");

			// 砲台位置を補正
			// 回転軸に合わせて移動
			groupCannons[0].getTransforms().add(new Translate( 1.8, 0.0, 0.0));
			groupCannons[1].getTransforms().add(new Translate( 0.4, 0.0, 0.0));
			groupCannons[2].getTransforms().add(new Translate(-3.4, 0.0, 0.0));

			// 元の位置へ移動
			groupCannons[0].setTranslateX(x - 1.0);
			groupCannons[1].setTranslateX(x - 0.2);
			groupCannons[2].setTranslateX(x + 1.7);

			super.setTranslateX(x);
			super.setTranslateY(y);
			super.setTranslateZ(z);

			// 回転軸を設定
			this.setRotationAxis(Rotate.Y_AXIS);

			// 砲台数分、ループ
			for(int i = 0; i < 3; i++){

				// 砲台をワールド座標系に再配置する
				this.getChildren().remove(groupCannons[i]);
				application.addNode(groupCannons[i]);

				// 縮小する
				groupCannons[i].setScaleX(scale);
				groupCannons[i].setScaleY(scale);
				groupCannons[i].setScaleZ(scale);

				// 
				groupCannons[i].setTranslateY(y);
				groupCannons[i].setTranslateZ(z);

				// 回転軸を設定
				groupCannons[i].setRotationAxis(Rotate.Y_AXIS);

			} // end for 砲台数分、ループ

			// グループ・メッシュビューを取得
			Group group = map.get("Body");
			MeshView meshView = (MeshView)(group.getChildren().get(0));

			// マテリアルを取得
			materail = (PhongMaterial)meshView.getMaterial();

			// 元の色を取得
			colorDefault = materail.getDiffuseColor();

			// 影を設定
			application.addShadow(this, 3.0);

			// ストーリーを停止する
			application.getPRStoryPrusher().stop();

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteBattleShip

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 時間＋１
		this.time++;

		// フェーズ＝０の場合、
		if(phase == 0){

			// 座標加算
			z = z + mz;

			// 移動
			super.setTranslateZ(z);

			// 停止位置に来た場合、
			if(z <= 40){

				// フェーズ１へ
				phase = 1;

				// 移動
				super.setTranslateZ(40);

				// ゴルBGMスタート
				application.startMP3BGMBattleShip();

			}

		// フェーズ＝２、３の場合、
		}else if(phase == 2 || phase == 3){

			// 回転
			angle = time / 3;

			// 回転が120を超えたら、
			if(phase == 2 && angle > 120){
				// フェーズ３へ
				phase = 3;
			}

			// 回転が280を超えたら、
			if(angle > 280){

				// フェーズ４へ
				phase = 4;

				// 速度設定
				mx = -0.04;
				mz = -0.2;

			}

			// 回転
			this.setRotate(angle);
			x = 10 * Math.cos((-angle - 90) * 3.141592 / 180);
			z = 60 + 20 * Math.sin((-angle - 90) * 3.141592 / 180);

			// 移動
			super.setTranslateX(x);
			super.setTranslateZ(z);

		// フェーズ＝４場合、
		}else if(phase == 4){

			// 座標加算
			x = x + mx;
			z = z + mz;

			// 移動
			super.setTranslateX(x);
			super.setTranslateZ(z);

			// 画面の外にでた場合、
			if(z <= -5.0){

				// ゴルBGMストップ
				application.stopMP3BGMBattleShip();

				// ストーリーを再開する
				application.getPRStoryPrusher().start();

				// 削除する
				super.remove();

			} // end if 画面の外にでた場合

		} // end if フェーズ

		// 砲台１が存在する場合、
		if(groupCannons[0] != null){

			// 砲台を取得
			Group groupCannon = groupCannons[0];

			// 位置を補正
			groupCannon.setTranslateX(x - 2.0 * Math.cos(angle * 3.141592 / 180));
			groupCannon.setTranslateY(y);
			groupCannon.setTranslateZ(z + 2.0 * Math.sin(angle * 3.141592 / 180));

			// 回転
			groupCannon.setRotate(angleCannons[0]);

		} // end if 砲台１が存在する場合

		// 砲台２が存在する場合、
		if(groupCannons[1] != null){

			// 砲台を取得
			Group groupCannon = groupCannons[1];

			// 位置を補正
			groupCannon.setTranslateX(x - 0.4 * Math.cos(angle * 3.141592 / 180));
			groupCannon.setTranslateY(y);
			groupCannon.setTranslateZ(z - 0.4 * Math.sin(angle * 3.141592 / 180));

			// 回転
			groupCannon.setRotate(angleCannons[1]);

		} // end if 砲台２が存在する場合

		// 砲台３が存在する場合、
		if(groupCannons[1] != null){

			// 砲台を取得
			Group groupCannon = groupCannons[2];

			// 位置を補正
			groupCannon.setTranslateX(x + 3.4 * Math.cos(angle * 3.141592 / 180));
			groupCannon.setTranslateY(y);
			groupCannon.setTranslateZ(z - 3.4 * Math.sin(angle * 3.141592 / 180));

			// 回転
			groupCannon.setRotate(angleCannons[2]);

		} // end if 砲台３が存在する場合

		// 砲台分、ループ
		for(int i = 0; i < 3; i++){

			// フェーズ毎−砲台が攻撃する場合、
			if(phase == 1 ||
				phase == 2 && i == 2 ||
				phase == 3 && (i == 0 || i == 1) ||
				phase == 4 && (i == 0 || i == 1)){

				// 砲台を取得
				Group groupCannon = groupCannons[i];

				// 砲台が存在しない場合、
				if(groupCannons[0] == null){
					// コンティニュー
					continue;
				}

				// 角度を取得
				double angleCannon = angleCannons[i];
				angleCannon = (angleCannon >= 0)?angleCannon: angleCannon + 360;
				angleCannon = (angleCannon <= 360)?angleCannon: angleCannon - 360;

				// 砲台フェーズ＝０の場合、
				if(phaseCannons[i] == 0){

					//
					// 砲台角度設定
					//

					// 座標取得
					double xCannon = groupCannon.getTranslateX();
					double zCannon = groupCannon.getTranslateZ();

					// プレイヤーを取得
					PRSpritePrusher spritePrusher = application.getPRSpritePrusher();

					// 座標取得
					double xPrusher = spritePrusher.getTranslateX();
					double zPrusher = spritePrusher.getTranslateZ();

					// ラジアン取得
					// Z方向は奥へ行くほどプラスになるので、計算上、逆にする
					double rad = Math.atan2(zCannon - zPrusher, xPrusher - xCannon);

					// 角度取得
					double angleP = rad * 180 / Math.PI;
					angleP = (angleP >= 0)?angleP: angleP + 360;
					angleP = (angleP <= 360)?angleP: angleP - 360;

					// 3度以内
					if(Math.abs(angleP - angleCannon) < 3.0){

						// 何もしない

					// 3度以上
					}else{

						// 角度差を求める
						double angleSa = angleP - angleCannon;

						// プラスの場合、
						if(angleSa > 0){
							// +2度
							angleCannon = angleCannon + 1;
						// 上記以外の場合、
						}else{
							// -2度
							angleCannon = angleCannon - 1;
						}

						// 角度補正
						angleCannon = (angleCannon >= 0)?angleCannon: angleCannon + 360;
						angleCannon = (angleCannon <= 360)?angleCannon: angleCannon - 360;

						// 角度を設定
						angleCannons[i] = angleCannon;

					} // end if

				} // end if 砲台フェーズ＝０の場合

				//
				// 発射制御
				//

				// 時間−１
				timeCannons[i]--;

				// 時間が経った場合、
				if(timeCannons[i] <= 0){

					// 砲台フェーズ＝０の場合、
					if(phaseCannons[i] == 0){

						// 敵の弾を生成
						PRSpriteGroupLaserTeki2 spriteGroupTamaTeki = new PRSpriteGroupLaserTeki2(application, groupCannon);
						// 敵の弾を追加
						application.addSpriteGroup(spriteGroupTamaTeki);

						// フェーズ＝１へ
						phaseCannons[i] = 1;

						// 時間初期化
						timeCannons[i] = 180 - ((phase == 2 || phase == 3 || phase == 4)?150: 0);

					// 砲台フェーズ＝１の場合、
					}else if(phaseCannons[i] == 1){

						// フェーズ＝２へ
						phaseCannons[i] = 2;

						// 時間初期化
						timeCannons[i] = 120 - ((phase == 2 || phase == 3 || phase == 4)?90: 0);

					// 砲台フェーズ＝２の場合、
					}else if(phaseCannons[i] == 2){

						// フェーズ＝３へ
						phaseCannons[i] = 3;

						// 時間初期化
						timeCannons[i] = 60;

					// 砲台フェーズ＝３の場合、
					}else if(phaseCannons[i] == 3){

						// フェーズ＝０へ
						phaseCannons[i] = 0;

						// 時間初期化
						timeCannons[i] = (int)(Math.random() * 60);

					} // end if 砲台フェーズ

				} // end if 時間が経った場合

			} // end if フェーズ毎−砲台が攻撃する場合

		} // end for 砲台分、ループ

		// フェーズ＝０、１、２、３の場合、
		if(phase == 0 || phase == 1 || phase == 2 || phase == 3){

			// 10％
			if(Math.random() < 0.10){

				// 敵の弾を生成
				PRSpriteTamaTeki3 spriteTama = new PRSpriteTamaTeki3(application, this, 1.0 - Math.random() * 2.0);
				// 敵の弾を追加
				application.addNode(spriteTama);

			} // end if 確率

		} // end if フェーズ

		// フェーズ＝０、１の場合、
		if(phase == 0 || phase == 1){

			// 時間が来たら
			if(time % 360 == 0){

				// アステロイダースプライトグループを生成
				PRSpriteGroupShip spriteGroup = new PRSpriteGroupShip(application, "ゴル援護");

				// アプリケーションにスプライトグループを追加
				application.addSpriteGroup(spriteGroup);

			}

		} // end if フェーズ

		//
		// 当たり判定
		//

		// フェーズ＝１の場合、
		if(phase == 1){

			// 境界を取得
			Bounds boundsThis = this.getBoundsInParent();

			// スプライト一覧を取得
			ObservableList<Node> nodeLst = application.getNodeLst();

			// スプライト一覧の件数を取得
			int countNodeLst = nodeLst.size();

			// スプライト一覧の件数分、ループ
			for(int i = 0; i < countNodeLst; i++){

				// スプライトを取得
				Node node = nodeLst.get(i);

				// 弾の場合、
				if(node instanceof PRSpriteTama){

					// 当たっている場合、
					if(boundsThis.intersects(node.getBoundsInParent())){

						// 弾へ変換
						PRSpriteTama spriteTama = (PRSpriteTama)node;

						// 跳ね返されている場合、
						if(spriteTama.isRapel()){
							// コンティニュー
							continue;
						}

						// 弾を跳ね返す
						spriteTama.rapel();

						// 耐久度−１
						life--;

						// 耐久度が１になった場合、
						if(life <= 1){

							// フェーズ＝２へ
							phase = 2;

							// 時間初期化
							time = 0;

						}

						// 一定値減った場合、
						if(life % 4 == 0){

							// 爆発音スタート
							application.startWaveBomb();

						}

						// ループを抜ける
						break;

					} // end if 当たっている場合

				} // end if 弾の場合

			} // end for スプライト一覧の件数分、ループ

			// 時間が来たら、
			if(life != LIFE && time % life < 2){

				// 装甲の色の設定
				materail.setDiffuseColor(COLOR_ARMOR);
				materail.setSpecularColor(COLOR_ARMOR);

			// 上記以外の場合、
			}else{

				// 色を元に戻す
				materail.setDiffuseColor(colorDefault);
				materail.setSpecularColor(colorDefault);

			}

		} // end if フェーズ＝１の場合

	} // end run

	/**
	 * 削除
	 * <pre>
	 * 砲台が上位グループに所属するようにしているため、
	 * 上位グループから砲台を削除する処理を追加しています。
	 * </pre>
	 */
	public void remove(){

		// 砲台数分、ループ
		for(int i = 0; i < 3; i++){
			// 砲台をワールド座標系より削除する
			application.removeNode(groupCannons[i]);
		}

		// スーパークラスの削除メソッド呼び出し
		super.remove();

	} // end remove

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.4;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.4;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 0.4;
	}

}
