import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Sphere;

/**
 * 爆発
 */
public class PRSpriteBomb extends PRSprite {

	/** 球の色 */
	private static final Color[]		COLORS			= new Color[]{
		Color.color(0.6, 0.6, 0.0, 0.2),
		Color.color(0.6, 0.6, 0.0, 0.4),
		Color.color(0.8, 0.8, 0.0, 0.6),
		Color.color(0.8, 0.8, 0.0, 0.8),
		Color.color(0.9, 0.9, 0.0, 1.0),
		Color.color(0.8, 0.8, 0.0, 0.8),
		Color.color(0.8, 0.8, 0.0, 0.6),
		Color.color(0.6, 0.6, 0.0, 0.4),
		Color.color(0.6, 0.6, 0.0, 0.2)
	};

	/** アプリケーション */
	public		PRApplication			application		= null;

	/** 爆発させるスプライト */
	public		PRSprite				sprite			= null;

	/** 球 */
	private		Sphere					sphere			= null;

	/** マテリアル */
	private		PhongMaterial			material		= null;

	/** 速度 */
	private		double					mx				= 0.0;
	private		double					my				= 0.0;
	private		double					mz				= 0.0;

	/** 時間 */
	private		int						time			= 0;

	/** 半径増分 */
	private		double					aRadius			= 0;

	/** 色位置 */
	private		int						posColors		= 0;

	/**
	 * コンストラクタ
	 * <pre>
	 * 爆発半径増分は0.02で設定されます。
	 * <pre>
	 * @param application PRApplication
	 * @param group Group 爆発させるスプライト
	 */
	public PRSpriteBomb(PRApplication application, PRSprite sprite) {

		this(application, sprite, 0.02);

	}

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param group Group 爆発させるスプライト
	 * @param aRadius 爆発半径増分
	 */
	public PRSpriteBomb(PRApplication application, PRSprite sprite, double aRadius) {

		super(application);

		// アプリケーションを保持
		this.application = application;

		// 爆発させるスプライトを保持
		this.sprite = sprite;

		// 場所を設定
		this.setTranslateX(sprite.getTranslateX());
		this.setTranslateY(sprite.getTranslateY());
		this.setTranslateZ(sprite.getTranslateZ() - 1);

		// 半径増分を計算
		this.aRadius = aRadius;

		// 球を生成
		sphere = new Sphere(0.0, 16);

		// マテリアルを生成
		material = new PhongMaterial(COLORS[0]);

		// マテリアルを設定
		sphere.setMaterial(material);

		// 敵スプライトの場合、
		if(sprite instanceof PRSpriteT){

			// 敵スプライトにキャスト
			PRSpriteT spriteT = (PRSpriteT)sprite;

			// 速度を設定
			this.mx = spriteT.mx;
			this.my = spriteT.my;
			this.mz = spriteT.mz;

		} // end if 敵スプライトの場合

		// 球を追加
		this.getChildren().add(sphere);

	} // end PRSpriteBomb

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 時間＋１
		time++;

		// 色位置が４未満の場合、
		if(posColors < 4){

			// 位置を変更
			// 爆発させるスプライトに合わせて爆発を移動
			super.setTranslateX(sprite.getTranslateX());
			super.setTranslateY(sprite.getTranslateY());
			super.setTranslateZ(sprite.getTranslateZ());

		}else{

			// 位置を変更
			// 爆発させるスプライトが削除されてしまってるので、取得した速度で移動
			super.setTranslateX(super.getTranslateX() + mx);
			super.setTranslateY(super.getTranslateY() + my);
			super.setTranslateZ(super.getTranslateZ() + mz);

		} // end if 位置を変更

		// 0.25秒
		if(time < 15){

			// 半径を設定
			sphere.setRadius(sphere.getRadius() + aRadius);

		// 0.5秒超
		}else{

			// 0.05秒単位の場合、
			if(time % 3 == 0){

				// 色位置＋１
				posColors++;

				// 色が無くなった場合、
				if(posColors >= COLORS.length){

					// アプリケーションからノード削除
					application.removeNode(this);

					// リターン
					return;

				}

				// 色を変更
				material.setDiffuseColor(COLORS[posColors]);

				// 色位置が４になった場合、
				if(posColors == 4){

					// 爆発させるスプライトを削除
					application.removeNode(sprite);

				}

			} // end if // 0.2秒単位の場合

		} // end if 0.5秒超

	} // end run

}
