import java.util.StringTokenizer;
import javafx.scene.shape.Sphere;
import javafx.scene.paint.PhongMaterial;
import javafx.geometry.Bounds;
import javafx.scene.paint.Color;

/**
 * 雲
 */
public class PRSpriteCloud extends PRSprite {

	/** マテリアル */
	private static		PhongMaterial	material	= null;

	/** ループフラグ */
	private				boolean			flgLoop		= true;

	/** X座標 */
	private				double			x			= 0.0;

	/** 元々のX */
	private				double			xDefault	= 0.0;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			// マテリアルを生成
			material = new PhongMaterial();
			material.setDiffuseColor(Color.color(0.8, 0.8, 0.8, 0.3));

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 */
	public PRSpriteCloud(PRApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// 大きさを設定
			double size = Math.random() * 10 + 5;

			// 雲の数を設定
			int count = (int)(size * 2);

			// 雲の数分、ループ
			for(int i = 0; i < count; i++){

				// 球を生成
				Sphere sphere = new Sphere((Math.random() / 2 + 0.5) * size * 0.10, 12);

				// マテリアルを登録
				sphere.setMaterial(material);

				// 球を追加
				this.getChildren().add(sphere);

				// 球の位置を設定
				sphere.setTranslateX((Math.random() - 0.50) * size * 0.50);
				sphere.setTranslateY((Math.random() - 0.50) * size * 0.05);
				sphere.setTranslateZ((Math.random() - 0.50) * size * 0.50);

			} // end for 雲の数分、ループ

			// X設定
			xDefault = Math.random() * 30.0 - 15.0;
			x = xDefault;
			super.setTranslateX(x);

			// Y設定
			Bounds bounds = this.getBoundsInParent();
			super.setTranslateY(bounds.getHeight() / 2);

			// Z設定
			super.setTranslateZ(200);

			// 速度設定
			mz = -2.0;

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteCloud

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param strParam 配置パターン
	 */
	public PRSpriteCloud(PRApplication application, String strParam){

		// スーパークラスを呼び出す
		super(application);

		try{

			//
			// パラメータから各要素を取得
			//

			// トークンを作成
			StringTokenizer stParam = new StringTokenizer(strParam, " ");

			// 大きさを取得
			double size = Double.parseDouble(stParam.nextToken());

			// X座標
			x = Double.parseDouble(stParam.nextToken());

			// Y座標
			double y = Double.parseDouble(stParam.nextToken());

			//
			// 雲を生成
			//

			// 雲の数を設定
			int count = (int)(size * 5);

			// 雲の数分、ループ
			for(int i = 0; i < count; i++){

				// 球を生成
				Sphere sphere = new Sphere((Math.random() / 2 + 0.5) * size * 0.10, 16);

				// マテリアルを登録
				sphere.setMaterial(material);

				// 球を追加
				this.getChildren().add(sphere);

				// 球の位置を設定
				sphere.setTranslateX((Math.random() - 0.50) * size * 0.50);
				sphere.setTranslateY((Math.random() - 0.50) * size * 0.10);
				sphere.setTranslateZ((Math.random() - 0.50) * size * 0.50);

			} // end for 弾の数分、ループ

			// X設定
			super.setTranslateX(x);

			// Y設定
			super.setTranslateY(y);

			// Z設定
			super.setTranslateZ(200);

			// 速度設定
			mz = -2.0;

			// ループはさせない
			flgLoop = false;

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteCloud

	/**
	 * 実行
	 */
	@Override
	public void run(){

		//
		// X方向
		//

		// 移動量取得
		double mx = -application.getPRSpritePrusher().mx;

		// 座標計算
		x = x + mx;

		// 移動
		super.setTranslateX(x);

		//
		// Z方向
		//

		// 座標取得
		double z = super.getTranslateZ();

		// 座標加算
		z = z + mz;

		// 画面の外にでた場合、
		if(z < 2){

			// ループさせる場合、
			if(flgLoop){

				// 位置を戻す
				x = xDefault;
				z = 200;

			// ループさせない場合、
			}else{

				// 削除する
				super.remove();

			}

		} // end if 画面の外にでた場合

		// 移動
		super.setTranslateZ(z);

	} // end run

	/**
	 * 終了
	 * <pre>
	 * 雲を終了させます。
	 * <pre>
	 */
	public void finish(){

		// ループ取り止め
		flgLoop = false;

	} // end finish

}
