import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Cylinder;
import javafx.scene.transform.Rotate;

/**
 * 敵のレーザー１
 */
public class PRSpriteLaserTeki1 extends PRSpriteTamaTeki0 {

	/** レーザーの色 */
	private static final Color[]	COLOR_LASER		= new Color[]{
		Color.color(1.0, 0.4, 0.1, 0.5),
		Color.color(1.0, 0.2, 0.1, 0.5),
		Color.color(1.0, 0.0, 0.1, 0.5)
	};

	/** 敵スプライト */
	private		PRSprite			sprite			= null;

	/** シリンダー */
	private		Cylinder			cylinder		= null;

	/** マテリアル */
	private		PhongMaterial		materail		= null;

	/** 時間 */
	private		int					time			= 0;

	/** 速度 */
	private		double				mx				= 0;
	private		double				my				= 0;
	private		double				mz				= 0;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param spritePrusher プレイヤー
	 * @param speed 敵の弾のスピード
	 * @param flgBoss ボスフラグ
	 */
	public PRSpriteLaserTeki1(PRApplication application, PRSprite sprite, boolean flgBoss) {

		super(application);

		// 敵スプライトを保持
		this.sprite = sprite;

		// シリンダーを生成
		cylinder = new Cylinder(0.25, 0.5, 12);

		// 回転させる（90度傾ける）
		cylinder.getTransforms().add(new Rotate(90, Rotate.X_AXIS));

		// シリンダーを追加
		this.getChildren().add(cylinder);

		// 座標
		double x = sprite.getTranslateX();
		double y = sprite.getTranslateY();
		double z = sprite.getTranslateZ() - 0.25;

		// 位置を設定
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// マテリアルを生成
		materail = new PhongMaterial();

		// マテリアルを設定
		cylinder.setMaterial(materail);

		// 鏡面色を設定
		materail.setSpecularPower(0.2);

		// ボスの場合、
		if(flgBoss){

			// 大きさを２倍にする
			super.setScaleX(2.0);
			super.setScaleY(2.0);

		}

		// 影を設定
		application.addShadow(this);

		// 速度を設定
		mz = -0.05;

	} // end PRSpriteLaserTeki1

	/**
	 * 実行
	 */
	public void run(){

		// 位置取得
		double x = super.getTranslateX();
		double y = super.getTranslateY();
		double z = super.getTranslateZ();

		// 移動
		x = x + mx;
		y = y + my;
		z = z + mz;

		// 画面の外にでた場合、
		if(z < -10){

			// 削除
			application.removeNode(this);

		}

		// １秒まで
		if(time < 60){

			// 敵スプライトに位置を合わせる
			x = sprite.getTranslateX();
			y = sprite.getTranslateY();

		// １秒
		}else if(time == 60){

			// 敵スプライトの保持を削除
			this.sprite = null;

		}

		// 位置設定
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// 時間設定
		time++;

		// １秒まで
		if(time < 60){

			// 高さを伸ばす
			cylinder.setHeight(cylinder.getHeight() + 0.1);

		// １秒
		}else if(time == 60){

			// 速度を設定
			mz = -1.0;

		} // end if 時間

		// 弾の色の設定
		materail.setDiffuseColor(COLOR_LASER[time % 3]);
		materail.setSpecularColor(COLOR_LASER[time % 3]);

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.4;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.4;
	}

	/**
	 * 当たり判定範囲Z
	 */
//	@Override
//	public double getHaniZ() {
//		return 2.0;
//	}

}
