import javafx.collections.ObservableList;
import javafx.geometry.Bounds;
import javafx.scene.Node;

/**
 * 柱
 */
public class PRSpritePole extends PRSpriteT {

	/** スプライト定義 */
	private static final String	POLE_WHITE		= "PRSpritePole_WHITE";
	private static final String	POLE_PURPLE		= "PRSpritePole_PURPLE";

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(POLE_WHITE, "3d/Pole_White.obj");
			ObjReader.init(POLE_PURPLE, "3d/Pole_Purple.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param strPattern 配置パターン
	 * @param stage ステージ
	 */
	public PRSpritePole(PRApplication application, String strPattern){

		// スーパークラスを呼び出す
		super(application);

		try{

			// オブジェクト
			if("白".equals(strPattern)){

				ObjReader.create(this, POLE_WHITE);

				// 位置設定
				double x = Math.random() * 24.0 - 12.0;
				super.setTranslateX(x);
				super.setTranslateY(0.00);
				super.setTranslateZ(150);

			}else if("紫".equals(strPattern)){

				ObjReader.create(this, POLE_PURPLE);

				// 位置設定
				double x = Math.random() * 24.0 - 12.0;
				super.setTranslateX(x);
				super.setTranslateY(0.00);
				super.setTranslateZ(150);

			}else if("紫左".equals(strPattern)){

				ObjReader.create(this, POLE_PURPLE);

				// 位置設定
				super.setTranslateX(-5.0);
				super.setTranslateY(0.0);
				super.setTranslateZ(150);

			}else if("紫右".equals(strPattern)){

				ObjReader.create(this, POLE_PURPLE);

				// 位置設定
				super.setTranslateX(5.0);
				super.setTranslateY(0.0);
				super.setTranslateZ(150);

			}

			// 速度設定
			mz = -1.0;

			// 影を設定
			application.addShadow(this, 0.6);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpritePole

	/**
	 * 実行
	 */
	@Override
	public void run(){

		//
		// X方向
		//

		// 移動量取得
		double mx = -application.getPRSpritePrusher().mx;

		// 座標計算
		double x = super.getTranslateX();
		x = x + mx;

		// 移動
		super.setTranslateX(x);

		//
		// Z方向
		//

		// 座標取得
		double z = super.getTranslateZ();

		// 座標加算
		z = z + mz;

		// 画面の外にでた場合、
		if(z < 0){
			// 削除する
			super.remove();
		}

		// 移動
		super.setTranslateZ(z);

		//
		// 当たり判定
		//

		// 境界を取得
		Bounds boundsThis = this.getBoundsInParent();

		// スプライト一覧を取得
		ObservableList<Node> nodeLst = application.getNodeLst();

		// スプライト一覧の件数を取得
		int countNodeLst = nodeLst.size();

		// スプライト一覧の件数分、ループ
		for(int i = 0; i < countNodeLst; i++){

			// スプライトを取得
			Node node = nodeLst.get(i);

			// 弾の場合、
			if(node instanceof PRSpriteTama){

				// 当たっている場合、
				if(boundsThis.intersects(node.getBoundsInParent())){

					// 弾へ変換
					PRSpriteTama spriteTama = (PRSpriteTama)node;

					// 跳ね返されている場合、
					if(spriteTama.isRapel()){
						// コンティニュー
						continue;
					}

					// 弾を跳ね返す
					spriteTama.rapel();

					// ループを抜ける
					break;

				} // end if 当たっている場合

			} // end if 弾の場合

		} // end for スプライト一覧の件数分、ループ

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.4;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.8;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 0.2;
	}

}
