import java.awt.event.*;
import java.util.HashMap;

import javafx.collections.ObservableList;
import javafx.geometry.BoundingBox;
import javafx.geometry.Bounds;
import javafx.geometry.Point3D;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.input.KeyCode;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.MeshView;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Translate;

/**
 * プレイヤー
 */
public class PRSpritePrusher extends PRSprite {

	/** ショットボタン割り当て */
	private int[]					btnShots			= null;

	/** フェーズ−スタート */
	private final int				PHASE_START			= 0;

	/** フェーズ−リスタート */
	private final int				PHASE_RESTART		= 1;

	/** フェーズ−無敵中 */
	private final int				PHASE_MUTEKI		= 2;

	/** フェーズ−ゲーム中 */
	private final int				PHASE_GAME			= 3;

	/** フェーズ−衝突 */
	private final int				PHASE_BUMP			= 4;

	/** フェーズ−ゲームクリア */
	private final int				PHASE_GAME_CLEAR	= 5;

	/** Y座標地面 */
	private		double				yJimen				= 0.0;

	/** フェーズ */
	private		int					phase				= PHASE_START;

	/** 時間 */
	private		int					time				= 0;

	/** 上移動 */
	private		KeyCode				moveUp				= KeyCode.DOWN;

	/** 下移動 */
	private		KeyCode				moveDown			= KeyCode.UP;

	/** 上移動 */
	private		int					moveUpPAD			= -1;

	/** 下移動 */
	private		int					moveDownPAD			= -1;

	/** 弾発射時間 */
	private		int					timeTama			= 0;

	/** 無敵状態 */
	private		boolean				flgMuteki			= false;

	/** 衝突状態 */
	private		boolean				flgBump				= false;

	/** 右腕 */
	private		Group				groupArmRight		= null;

	/** ファイアー */
	private		Group				groupFire			= null;

	/** 左腕 */
	private		Group				groupArmLeft		= null;

	/** 右足 */
	private		Group				groupLegRight		= null;

	/** 左足 */
	private		Group				groupLegLeft		= null;

	/** 足・腕の制御 */
	private		boolean				flgJimen			= true;
	private		int					angleLeg			= 0;
	private		int					dirLeg				= 1;

	/** 位置 */
	private		double				x					= 0.0;
	private		double				y					= 0.0;

	/** 飛んでいる感を出す */
	private		double				xx					= 0.0;
	private		double				yy					= 0.0;

	/** 移動量 */
	public		double				mx					= 0.0;
	public		double				my					= 0.0;

	/** 無反動砲のマテリアル */
	private		PhongMaterial		materail			= null;

	/** 回転軸X */
	private		Point3D				px					= new Point3D(1.0, 0.0, 0.0);

	/** 回転軸Y */
	private		Point3D				py					= new Point3D(0.0, 1.0, 0.0);

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpritePrusher.class, "3d/Prusher.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 */
	public PRSpritePrusher(PRApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// オブジェクト生成
			HashMap<String, Group> map = ObjReader.create(this);

			// 各パーツを保持
			groupArmRight = map.get("Arm_Right");
			groupFire = map.get("Fire");
			groupArmLeft = map.get("Arm_Left");
			groupLegRight = map.get("Leg_Right_Leg");
			groupLegLeft = map.get("Leg_Left_Leg");

			// 前傾姿勢にする
			this.getTransforms().add(new Rotate(-10, Rotate.X_AXIS));

			// 地面に接触するようにY座標を修正する
			Bounds bounds = this.getBoundsInParent();
			yJimen = -bounds.getHeight() / 2;
			y = yJimen;

			// 移動させる
			groupArmRight.setTranslateX(groupArmRight.getTranslateX() - 0.042);
			groupFire.setTranslateX(groupFire.getTranslateX() - 0.042);
			groupArmLeft.setTranslateX(groupArmLeft.getTranslateX() + 0.042);
			groupLegRight.setTranslateX(groupLegRight.getTranslateX() - 0.21);
			groupLegLeft.setTranslateX(groupLegLeft.getTranslateX() + 0.21);

			groupArmRight.setTranslateY(groupArmRight.getTranslateY() - 0.112);
			groupFire.setTranslateY(groupFire.getTranslateY() - 0.112);
			groupArmLeft.setTranslateY(groupArmLeft.getTranslateY() - 0.112);
			groupLegRight.setTranslateY(groupLegRight.getTranslateY() + 0.04);
			groupLegLeft.setTranslateY(groupLegLeft.getTranslateY() + 0.04);

			groupLegRight.setTranslateZ(groupLegRight.getTranslateZ() - 0.04);
			groupLegLeft.setTranslateZ(groupLegLeft.getTranslateZ() - 0.04);

			// 中心点設定
			groupArmRight.getTransforms().add(new Translate(0.0, 0.11, 0.0));
			groupFire.getTransforms().add(new Translate(0.0, 0.11, 0.0));
			groupArmLeft.getTransforms().add(new Translate(0.0, 0.11, 0.0));
			groupLegRight.getTransforms().add(new Translate(0.0, -0.04, 0.04));
			groupLegLeft.getTransforms().add(new Translate(0.0, -0.04, 0.04));

			// 回転軸を設定
			groupArmRight.setRotationAxis(px);
			groupFire.setRotationAxis(px);
			groupArmLeft.setRotationAxis(px);
			groupLegRight.setRotationAxis(px);
			groupLegLeft.setRotationAxis(px);

			// 位置を設定
			this.setTranslateX(0.0);
			this.setTranslateY(y);
			this.setTranslateZ(6.0);

			// 無反動砲のメッシュビューを取得
			MeshView meshView = (MeshView)(groupFire.getChildren().get(0));

			// 無反動砲のマテリアルを取得
			materail = (PhongMaterial)meshView.getMaterial();

			// ショットボタン取得
			btnShots = application.getBtnShots();

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpritePrusher

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 時間＋１
		time++;

		// フェーズ−スタート
		if(phase == PHASE_START){

			// 回転
			super.setRotate(0);

			// 回転軸を設定
			super.setRotationAxis(py);

			// 時間を初期化
			time = 0;

			// フェーズ−リスタート
			phase = PHASE_RESTART;

		// フェーズ−リスタート
		}else if(phase == PHASE_RESTART){

			// 回転
			super.setRotate(0);

			// 回転軸を設定
			super.setRotationAxis(py);

			// 地面にいる場合、
			if(y == yJimen){
				// 地面にいる
				flgJimen = true;
			}

			// 無敵状態
			flgMuteki = true;

			// 衝突状態
			flgBump = false;

			// フェーズ−無敵中
			phase = PHASE_MUTEKI;

		// フェーズ−無敵中
		}else if(phase == PHASE_MUTEKI){

			// ３．０秒
			if(time >= 180){

				// 時間を初期化
				time = 0;

				// フェーズ−ゲーム中
				phase = PHASE_GAME;

			}

		// フェーズ−ゲーム中
		}else if(phase == PHASE_GAME){

			// 移動量初期化
			mx = 0.0;
			my = 0.0;

			// 上移動の場合、
			if(application.isKeyPressed(moveUp)  || application.isPAD() && InputControl.isKeyPressed(moveUpPAD)){

				// 座標計算
				my = my - 0.06;

			} // end if 上移動の場合

			// 下移動の場合、
			if(application.isKeyPressed(moveDown) || application.isPAD() && InputControl.isKeyPressed(moveDownPAD)){

				// 座標計算
				my = my + 0.06;

			} // end if 下移動の場合

			// 左が押された場合、
			if(application.isKeyPressed(KeyCode.LEFT) || application.isPAD() && InputControl.isKeyPressed(KeyEvent.VK_LEFT)){

				// 移動量計算
				mx = mx - 0.06;

			} // end if 左が押された場合

			// 右が押された場合、
			if(application.isKeyPressed(KeyCode.RIGHT) || application.isPAD() && InputControl.isKeyPressed(KeyEvent.VK_RIGHT)){

				// 移動量計算
				mx = mx + 0.06;

			} // end if 右が押された場合

			// Y移動量が設定された場合、
			if(my != 0.0){
				// Y方向移動
				moveY(my);
			} // end if Y移動量が設定された場合

			// X移動量が設定された場合、
			if(mx != 0.0){
				// X方向移動
				moveX(mx);
			}

			// 移動していない場合、
			if(mx == 0){
				// 回転
				super.setRotate(0);
			// 左方向に移動中の場合、
			}else if(mx < 0){
				// 回転
				super.setRotate(-15);
			// 右方向に移動中の場合、
			}else if(mx > 0){
				// 回転
				super.setRotate(15);
			}

			// 地面にいない または ステージ４ または ステージ５の場合、
			if(!flgJimen || super.application.getStage() == 4|| super.application.getStage() == 5){

				// 時間の場合、
				if(time % 5 == 0){

					// 飛んでる感を出す
					xx = Math.random() * 0.02 - 0.01;
					yy = Math.random() * 0.02 - 0.01;

					// 座標を設定
					this.setTranslateX(x + xx);
					this.setTranslateY(y + yy);

				} // end if 時間の場合

			} // end if 地面にいない かつ 時間の場合

			// スペース または Z または ショット（パッド）が押された場合、
			if(application.isKeyPressed(KeyCode.SPACE) || application.isKeyPressed(KeyCode.Z) ||
							application.isPAD() && InputControl.isKeyPressed(btnShots)){

				// 時間
				if(timeTama % 30 == 0){

					// ロックオンしている敵スプライトを取得
					PRSpriteT spriteT = application.getPRLock().getLockOnSpriteT();

					// 球を生成
					PRSpriteTama spriteTama = new PRSpriteTama(application, this, spriteT);

					// 弾を追加
					application.addNode(spriteTama);

					// ショット音スタート
					application.startWaveShot();

					// 無反動砲の色の設定
					materail.setDiffuseColor(Color.ORANGE);

				// 時間
				}else if(timeTama % 30 == 3){

					// 無反動砲の色の設定
					materail.setDiffuseColor(Color.RED);

				}

				// 時間＋１
				timeTama++;

			// スペース も Z も ショット（パッド） も押されていない場合、
			}else{

				// 弾時間を初期化
				timeTama = 0;

				// 無反動砲の色の設定
				materail.setDiffuseColor(Color.RED);

			}

			//
			// 当たり判定
			//

			// 境界を取得
			Bounds boundsThis = this.getBoundsInParent();

			// 当たり判定が厳しいので、少し緩めにする
			// Boundsは、protectedで生成できないので、BoundingBoxで生成する
			boundsThis = new BoundingBox(
				boundsThis.getMinX() + 0.15,
				boundsThis.getMinY() + 0.2,
				boundsThis.getMinZ(),
				boundsThis.getWidth() - 0.3,
				boundsThis.getHeight() -0.4,
				boundsThis.getDepth() + 0.4
			);

			// スプライト一覧を取得
			ObservableList<Node> nodeLst = application.getNodeLst();

			// スプライト一覧の件数を取得
			int countNodeLst = nodeLst.size();

			// スプライト一覧の件数分、ループ
			for(int i = 0; i < countNodeLst; i++){

				// ノードを取得
				Node node = nodeLst.get(i);

				// 敵スプライト以外の場合、
				if(!(node instanceof PRSpriteT)){
					// コンティニュー
					continue;
				}

				// 敵スプライトに変換
				PRSpriteT spriteT = (PRSpriteT)node;

				// 境界を取得
				Bounds boundsT = node.getBoundsInParent();

				// 奥行を取得
				double depth = boundsT.getDepth();

				// 奥行きがないスプライトの場合、
				// ２次元スプライトの場合、
				if(depth == 0.0){
					// 奥行を幅とする
					depth = boundsT.getWidth();
				}

				// 当たり判定が厳しいので、少し緩めにする
				// Boundsは、protectedで生成できないので、BoundingBoxで生成する
				Bounds boundsT2 = new BoundingBox(
					boundsT.getMinX() + boundsT.getWidth() * (0.5 - spriteT.getHaniX() / 2),
					boundsT.getMinY() + boundsT.getHeight() * (0.5 - spriteT.getHaniY() / 2),
					boundsT.getMinZ() + depth * (0.5 - spriteT.getHaniZ() / 2),
					boundsT.getWidth() * spriteT.getHaniX(),
					boundsT.getHeight() * spriteT.getHaniY(),
					depth * spriteT.getHaniZ()
				);

				// 当たっている場合、
				if(boundsThis.intersects(boundsT2)){

					// 敵が低木の場合、
					if(spriteT instanceof PRSpriteShrub){
						// コンティニュー
						continue;
					}

					// 無敵モードの場合、
					if(flgMuteki){
						// コンティニュー
						continue;
					}

					// 敵が爆発中の場合、
					if(spriteT.isBomb()){
						// コンティニュー
						continue;
					}

					// 敵がゴルの場合、
					if(spriteT instanceof PRSpriteBattleShip){
						// コンティニュー
						continue;
					}

					// 衝突
					bump();

				}

			} // end for スプライト一覧の件数分、ループ

		// フェーズ−衝突
		}else if(phase == PHASE_BUMP){

			// 0.25秒
			if(time < 15){

				// 何もしない

			// 0.75秒
			}else if(time < 45){

				// 移動
				this.moveY(-0.01);

				// 0.5秒
				if(time == 30){

					// 回転を戻す
					super.setRotate(0);

					// 回転軸を設定
					super.setRotationAxis(px);

					// 回転
					super.setRotate(90);
					groupArmRight.setRotate(90);
					groupFire.setRotate(90);
					groupArmLeft.setRotate(90);
					groupLegRight.setRotate(90);
					groupLegLeft.setRotate(90);

				} // end if 0.5秒

			// 1秒
			}else if(time == 60){

				// 回転軸を設定
				super.setRotationAxis(px);

				// 回転
				super.setRotate(180);

				groupArmRight.setRotate(180);
				groupFire.setRotate(180);
				groupArmLeft.setRotate(180);
				groupLegRight.setRotate(0);
				groupLegLeft.setRotate(0);

			// 1.5秒
			}else if(time < 90){

				// 移動
				this.moveY(0.01);

			// 2.5秒
			}else if(time < 150){

				// 移動
				this.moveY(0.02);

			}

		// フェーズ− ゲームクリア の場合、
		}else if(phase == PHASE_GAME_CLEAR){

			// 移動
			this.setTranslateZ(6.0 + time);

		} // end if フェーズ

		// フェーズ− 無敵中 または ゲーム中で２秒、
		if(phase == PHASE_MUTEKI || phase == PHASE_GAME && time <= 120){

			// 点滅
			if(time % 25 < 10){
				this.setVisible(false);
			}else{
				this.setVisible(true);
			}

			// フェーズ− 無敵中 かつ ２．８秒から３秒まで、
			if(phase == PHASE_MUTEKI && 168 <= time && time <= 180){

				// 上へ移動
				moveY(-0.1);

			// ゲーム中で２秒、
			}else if(phase == PHASE_GAME && time == 120){

				// 表示
				this.setVisible(true);

				// 無敵状態解除
				flgMuteki = false;

			} // end if ゲーム中で２秒

		} // end if  フェーズ− 無敵中 または ゲーム中で２秒

		// フェーズ− 無敵中 または ゲーム中 の場合、
		if(phase == PHASE_MUTEKI || phase == PHASE_GAME){

			// ステージ４、５以外の場合、
			if(super.application.getStage() != 4 && super.application.getStage() != 5){

				// 地面にいる場合、
				if(flgJimen){

					// 角度を計算
					angleLeg = angleLeg + 10 * dirLeg;

					if(Math.abs(angleLeg) > 60){
						// 方向を反転する
						dirLeg = -dirLeg;
					}

					groupArmRight.setRotate(angleLeg * 1.5);
					groupFire.setRotate(angleLeg * 1.5);
					groupArmLeft.setRotate(-angleLeg * 1.5);
					groupLegRight.setRotate(-angleLeg);
					groupLegLeft.setRotate(angleLeg);

				// 地面にいない場合、
				}else{

					groupArmRight.setRotate(180);
					groupFire.setRotate(180);
					groupArmLeft.setRotate(0);
					groupLegRight.setRotate(0);
					groupLegLeft.setRotate(0);

				} // end if 地面にいない場合

			// ステージ４の場合、
			}else{

				groupArmRight.setRotate(180);
				groupFire.setRotate(180);
				groupArmLeft.setRotate(0);
				groupLegRight.setRotate(0);
				groupLegLeft.setRotate(0);

			} // end if ステージ

		} // end if フェーズ− 無敵中 または ゲーム中 の場合

	} // end run

	/**
	 * Y方向移動
	 * <pre>
	 * 移動と着地の確認をしています。
	 * </pre>
	 * @param my Y移動量
	 */
	public void moveY(double my){

		// 移動
		double y1 = y + my;

		// 領域外の場合、
		if(y1 < -4.0){

			// 戻す
			y1 = -4.0;
			my = 0.0;

		// 地面ちょうどの場合、
		}else if(y1 == yJimen){

			// 地面にいる
			flgJimen = true;

		// 領域外の場合、
		}else if(y1 > yJimen){

			// 移動量を補正
			my = yJimen - y;
			y1 = yJimen;

			// 地面にいる
			flgJimen = true;

		// 上記以外の場合、
		}else{

			// 地面にいない
			flgJimen = false;

		} // end if

		// 座標を設定
		y = y1;
		this.setTranslateY(y);

		// カメラを移動
		application.moveCameraY(my / 1.5);

	} // end moveY

	/**
	 * X方向移動
	 * @param mx X移動量
	 */
	public void moveX(double mx){

		// 背景を移動
		application.moveSkyX(-mx * 40);

		// 移動
		x = x + mx;

		// 領域外の場合、
		if(x < -1.5){
			// 戻す
			x = -1.5;
		// 領域外の場合、
		}else if(x > 1.5){
			// 戻す
			x = 1.5;
		}

		// 座標を設定
		this.setTranslateX(x);

	} // end moveX

	/**
	 * 無敵状態取得
	 */
	public boolean isMuteki(){
		return flgMuteki;
	}

	/**
	 * 衝突状態取得
	 */
	public boolean isBump(){
		return flgBump;
	}

	/**
	 * スタート
	 */
	public void start(){

		// フェーズ−スタートへ
		phase = PHASE_START;

	} // end start

	/**
	 * リスタート
	 */
	public void restart(){

		// フェーズ−スタートへ
		phase = PHASE_RESTART;

	} // end start

	/**
	 * プレイヤーの位置を戻す
	 */
	public void initPos(){

		// 位置を戻す
		x = 0.0;
		this.setTranslateX(x);
		y = yJimen;
		this.setTranslateY(yJimen);
		this.setTranslateZ(6.0);

	} // end initPos

	/**
	 * 衝突
	 */
	public void bump(){

		// 時間を初期化
		time = 0;

		// 衝突状態
		flgBump = true;

		// フェーズ−衝突へ
		phase = PHASE_BUMP;

		// アプリケーションに衝突を伝える
		application.bump();

	} // end clash

	/**
	 * ゲームクリア
	 */
	public void clearGame(){

		// 時間を初期化
		time = 0;

		// フェーズ−ゲームクリアへ
		phase = PHASE_GAME_CLEAR;

	} // end clearGame

	/**
	 * モード設定
	 */
	public void setMode(int mode){

		// モード設定
		if(mode == 0){
			moveUp = KeyCode.DOWN;
			moveDown = KeyCode.UP;
			moveUpPAD = java.awt.event.KeyEvent.VK_DOWN;
			moveDownPAD = java.awt.event.KeyEvent.VK_UP;
		}else{
			moveUp = KeyCode.UP;
			moveDown = KeyCode.DOWN;
			moveUpPAD = java.awt.event.KeyEvent.VK_UP;
			moveDownPAD = java.awt.event.KeyEvent.VK_DOWN;
		}

	} // end run

}
