import javafx.geometry.Point3D;
import javafx.collections.ObservableList;
import javafx.scene.Node;
import javafx.geometry.Bounds;

/**
 * 岩
 */
public class PRSpriteRock extends PRSpriteT {

	/** 回転速度 */
	private		double			s			= Math.random() * 4 + 2;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteRock.class, "3d/Rock.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param stage ステージ
	 */
	public PRSpriteRock(PRApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// X設定
			double x = Math.random() * 24.0 - 12.0;

			// X設定
			super.setTranslateX(x);

			// Z設定
			super.setTranslateZ(100);

			// オブジェクト
			ObjReader.create(this);

			// 回転軸
			Point3D p = new Point3D(Math.random() - 0.5, Math.random() - 0.5, Math.random() - 0.5);

			// 回転軸を設定
			super.setRotationAxis(p);

			// 拡大／縮小する
			super.setScaleX(0.25);
			super.setScaleY(0.25);
			super.setScaleZ(0.25);

			// 速度設定
			mz = -0.6;

			// 影を設定
			application.addShadow(this, 0.6);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteRock

	/**
	 * 実行
	 */
	@Override
	public void run(){

		//
		// X方向
		//

		// 移動量取得
		double mx = -application.getPRSpritePrusher().mx;

		// 座標計算
		double x = super.getTranslateX();
		x = x + mx;

		// 移動
		super.setTranslateX(x);

		//
		// Z方向
		//

		// 座標取得
		double z = super.getTranslateZ();

		// 座標加算
		z = z + mz;

		// 画面の外にでた場合、
		if(z < 0){
			// 削除する
			super.remove();
		}

		// 移動
		super.setTranslateZ(z);

		// 回転
		super.setRotate(getRotate() + this.s);

		//
		// 当たり判定
		//

		// 爆発していない場合、
		if(!flgBomb){

			// 境界を取得
			Bounds boundsThis = this.getBoundsInParent();

			// スプライト一覧を取得
			ObservableList<Node> nodeLst = application.getNodeLst();

			// スプライト一覧の件数を取得
			int countNodeLst = nodeLst.size();

			// スプライト一覧の件数分、ループ
			for(int i = 0; i < countNodeLst; i++){

				// スプライトを取得
				Node node = nodeLst.get(i);

				// 弾の場合、
				if(node instanceof PRSpriteTama){

					// 当たっている場合、
					if(boundsThis.intersects(node.getBoundsInParent())){

						// 爆発
						flgBomb = true;

						// 弾を削除
						application.removeNode(node);

						// 爆発を生成
						PRSpriteBomb spriteBomb = new PRSpriteBomb(application, this);

						// アプリケーションに爆発を追加
						application.addNode(spriteBomb);

						// 点数追加
						application.addScore(100);

						// 爆発音スタート
						application.startWaveBomb();

						// ループを抜ける
						break;

					} // end if 当たっている場合

				} // end if 弾の場合

			} // end for スプライト一覧の件数分、ループ

		} // end if 爆発していない場合

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 0.4;
	}

}
