import javafx.collections.ObservableList;
import javafx.geometry.Bounds;
import javafx.scene.Node;
import javafx.scene.transform.Rotate;

/**
 * アステロイダー
 * <pre>
 * 結局、岩が好きなアステロイダー？！追いかけてきた？！？！
 * </pre>
 */
public class PRSpriteShip extends PRSpriteT {

	/** 攻撃パターン 右から */
	public static final int		POS_RIGHT		= PRSpriteGroupShip.POS_RIGHT;

	/** 攻撃パターン 左から */
	public static final int		POS_LEFT		= PRSpriteGroupShip.POS_LEFT;

	/** 配置パターン */
	private		String			strPattern		= null;

	/** 敵の弾発射状態 */
	private		boolean			flgTamaTeki		= false;

	/** 攻撃位置 */
	private 	int				pos				= 0;

	/** 時間 */
	private 	int				time			= 0;

	/** 傾きY */
	private		Rotate			rotateY			= null;
	private		double			angleY			= 180;

	/** 傾きZ */
	private		Rotate			rotateZ			= null;
	private		double			angleZ			= 0;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteShip.class, "3d/Ship.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param stage ステージ
	 * @param strPattern 配置パターン
	 */
	public PRSpriteShip(PRApplication application, String strPattern){

		// スーパークラスを呼び出す
		super(application);

		try{

			// パターンを保持
			this.strPattern = strPattern;

			// Z設定
			super.setTranslateZ(200);

			// パターン別に設定
			if("左上届かない".equals(strPattern)){

				// X設定
				super.setTranslateX(-10.0);
				// Y設定
				super.setTranslateY(-10.0);

				// 移動設定
				mz = -0.6;

			}else if("右上届かない".equals(strPattern)){

				// X設定
				super.setTranslateX(10.0);
				// Y設定
				super.setTranslateY(-10.0);

				// 移動設定
				mz = -0.6;

			}else if("左上届く".equals(strPattern)){

				// X設定
				super.setTranslateX(-2.0);
				// Y設定
				super.setTranslateY(-3.0);

				// 移動設定
				mz = -0.6;

			}else if("右上届く".equals(strPattern)){

				// X設定
				super.setTranslateX(2.0);
				// Y設定
				super.setTranslateY(-3.0);

				// 移動設定
				mz = -0.6;

			}else if("攻撃".equals(strPattern)){

				// X設定
				super.setTranslateX(Math.random() * 8.0 + 10.0);

				// Y設定
				super.setTranslateY(Math.random() * 2.0 - 12.0);

				// 移動設定
				mx = -0.12;
				my =  0.06;
				mz = -0.88;

			} // end if パターン別に設定

			// オブジェクト
			ObjReader.create(this);

			// 拡大／縮小する
			super.setScaleX(0.5);
			super.setScaleY(0.5);
			super.setScaleZ(0.5);

			// 影を設定
			application.addShadow(this, 0.8);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteShip

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param stage ステージ
	 * @param strPattern 配置パターン
	 * @param pos 攻撃位置
	 */
	public PRSpriteShip(PRApplication application, String strPattern, int pos){

		// スーパークラスを呼び出す
		super(application);

		try{

			// パターンを保持
			this.strPattern = strPattern;

			// 攻撃位置を保持
			this.pos = pos;

			// 後ろから攻撃 または ゴル援護 の場合、
			if("後ろから攻撃".equals(strPattern) || "ゴル援護".equals(strPattern)){

				// Rotateを生成
				rotateY = new Rotate(angleY, Rotate.Y_AXIS);

				// 回転用に登録
				this.getTransforms().add(rotateY);

				// Rotateを生成
				rotateZ = new Rotate(angleZ, Rotate.Z_AXIS);

				// 回転用に登録
				this.getTransforms().add(rotateZ);

				// Y設定
				super.setTranslateY(-2.0);

				// ゴル援護 の場合、
				if("ゴル援護".equals(strPattern)){
					// Y設定
					super.setTranslateY(-4.5);
				}

				// Z設定
				super.setTranslateZ(0);

				// 攻撃位置 右から、
				if(pos == POS_RIGHT){

					// X設定。右から出現させる
					super.setTranslateX(4.0);

				// 攻撃位置 左から、
				}else{

					// X設定。左から出現させる
					super.setTranslateX(-4.0);

				}

				// 移動設定
				mz =  1.00;

			} // end if パターン別に設定

			// オブジェクト
			ObjReader.create(this);

			// 拡大／縮小する
			super.setScaleX(0.5);
			super.setScaleY(0.5);
			super.setScaleZ(0.5);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteShip

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 座標取得
		double x = super.getTranslateX();
		double y = super.getTranslateY();
		double z = super.getTranslateZ();

		// 座標加算
		x = x + mx;
		y = y + my;
		z = z + mz;

		// 画面の外にでた場合、
		if(z < 0 || z > 200){
			// 削除する
			super.remove();
		}

		// パターン別に設定
		if("攻撃".equals(strPattern)){

			// Zが100を切った場合、
			if(z < 100){

				// まっすぐの飛行に切り替え
				mx = 0;
				my = 0;

				// 敵の弾を発射していない場合、
				if(!flgTamaTeki){

					// 敵の弾発射
					flgTamaTeki = true;

					// 敵の弾を生成
					PRSpriteTamaTeki1 spriteTamaTeki = new PRSpriteTamaTeki1(application, this, 1.0);
					// 敵の弾を追加
					application.addNode(spriteTamaTeki);

					// 敵の弾音スタート
					application.startWaveTamaTeki();

				} // end if 敵の弾を発射していない場合

			} // end if Zが100を切った場合、

		// 後ろから攻撃 または ゴル援護 の場合、
		}else if("後ろから攻撃".equals(strPattern) || "ゴル援護".equals(strPattern)){

			// 時間＋１
			time++;

			// 1.5秒
			if(time < 90){

				// 何もしない

			// 2.0秒
			}else if(time < 120){

				// 回転
				angleZ = angleZ + ((pos == POS_RIGHT)?3: -3);

				// 角度を設定
				rotateZ.setAngle(angleZ);

			// 2.0秒
			}else if(time == 120){

				// 移動設定
				mx = (pos == POS_RIGHT)?-0.3: 0.3;

				// 敵の弾を生成
				PRSpriteTamaTeki1 spriteTamaTeki = new PRSpriteTamaTeki1(application, this, 1.2);
				// 敵の弾を追加
				application.addNode(spriteTamaTeki);

				// 敵の弾音スタート
				application.startWaveTamaTeki();

			// 2.5秒
			}else if(time < 150){

				// 回転
				angleY = angleY + ((pos == POS_RIGHT)?-3: 3);

				// 角度を設定
				rotateY.setAngle(angleY);

			// 2.5秒
			}else if(time == 150){

				// 移動設定
				mz = 0;

			// 3.0秒
			}else if(time < 180){

				// 何もしない

			// 3.5秒
			}else if(time < 210){

				// 回転
				angleY = angleY + ((pos == POS_RIGHT)?-3: 3);

				// 角度を設定
				rotateY.setAngle(angleY);

			// 3.5秒
			}else if(time == 210){

				// 移動設定
				mz = -1.0;

			// 4.0秒
			}else if(time == 240){

				// 移動設定
				mx = (pos == POS_RIGHT)?0.3: -0.3;

				// 敵の弾を生成
				PRSpriteTamaTeki1 spriteTamaTeki = new PRSpriteTamaTeki1(application, this, 1.2);
				// 敵の弾を追加
				application.addNode(spriteTamaTeki);

				// 敵の弾音スタート
				application.startWaveTamaTeki();

			} // end if 時間

		} // end if パターン別に設定

		// 移動
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		//
		// 当たり判定
		//

		// 爆発していない場合、
		if(!flgBomb){

			// 境界を取得
			Bounds boundsThis = this.getBoundsInParent();

			// スプライト一覧を取得
			ObservableList<Node> nodeLst = application.getNodeLst();

			// スプライト一覧の件数を取得
			int countNodeLst = nodeLst.size();

			// スプライト一覧の件数分、ループ
			for(int i = 0; i < countNodeLst; i++){

				// スプライトを取得
				Node node = nodeLst.get(i);

				// 弾の場合、
				if(node instanceof PRSpriteTama){

					// 当たっている場合、
					if(boundsThis.intersects(node.getBoundsInParent())){

						// 爆発
						flgBomb = true;

						// 弾を削除
						application.removeNode(node);

						// 爆発を生成
						PRSpriteBomb spriteBomb = new PRSpriteBomb(application, this, 0.05);

						// アプリケーションに爆発を追加
						application.addNode(spriteBomb);

						// 点数追加
						application.addScore(500);

						// 爆発音スタート
						application.startWaveBomb();

						// ループを抜ける
						break;

					} // end if 当たっている場合

				} // end if 弾の場合

			} // end for スプライト一覧の件数分、ループ

		} // end if 爆発していない場合

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.5;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.5;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 0.6;
	}

}
