import javafx.scene.Group;
import javafx.scene.shape.MeshView;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.paint.Color;

/**
 * 弾
 */
public class PRSpriteTama extends PRSprite {

	/** 弾の色 */
	private static final Color[]	COLOR_TAMA		= new Color[]{
		Color.color(0.8, 0.8, 0.6, 0.5),
		Color.color(0.6, 0.6, 0.3, 0.5),
		Color.color(0.4, 0.4, 0.0, 0.5),
		Color.color(0.0, 0.0, 0.0, 0.5),
		Color.color(0.4, 0.4, 0.0, 0.5),
		Color.color(0.6, 0.6, 0.3, 0.5),
		Color.color(0.8, 0.8, 0.6, 0.5)
	};

	/** マテリアル */
	private		PhongMaterial		materail		= null;

	/** 時間 */
	private		int					time			= 0;

	/** 跳ね返された */
	private		boolean				flgRapel		= false;

	/** 跳ね返された時のX移動量 */
	private		double				mx				= 0;

	/** 跳ね返された時のY移動量 */
	private		double				my				= 0;

	/** ロックした敵スプライト */
	private		PRSpriteT			spriteT			= null;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteTama.class, "3d/Tama.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param spritePrusher プレイヤー
	 * @param spriteT 敵スプライト
	 */
	public PRSpriteTama(PRApplication application, PRSpritePrusher spritePrusher, PRSpriteT spriteT) {

		super(application);

		// オブジェクト生成
		ObjReader.create(this);

		// 敵スプライトを保持
		this.spriteT = spriteT;

		// 位置を設定
		super.setTranslateX(spritePrusher.getTranslateX());
		super.setTranslateY(spritePrusher.getTranslateY() - 0.1);
		super.setTranslateZ(spritePrusher.getTranslateZ() + 1.0);

		// グループ・メッシュビューを取得
		Group group = (Group)this.getChildren().get(0);
		MeshView meshView = (MeshView)(group.getChildren().get(0));

		// マテリアルを取得
		materail = (PhongMaterial)meshView.getMaterial();

	} // end PRSpriteTama

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 跳ね返されていない場合、
		if(!flgRapel){

			// 位置取得
			double z = super.getTranslateZ();

			// 移動
			z = z + 2;

			// 飛んでったら
			if(z > 200){

				// 削除
				application.removeNode(this);

				// リターン
				return;

			}

			// ロックオンしている場合、
			if(spriteT != null){

				// 爆発している場合、
				if(spriteT.isBomb()){

					// ロックオンを外す
					spriteT = null;

				// 爆発していない場合、
				}else{

					// 敵スプライトの位置に弾を合わせる
					super.setTranslateX(spriteT.getTranslateX());
					super.setTranslateY(spriteT.getTranslateY());

				} // end if

			} // end if ロックオンしている場合

			// 位置設定
			super.setTranslateZ(z);

			// 時間設定
			time++;
			time = (time >= 7)?0: time;

			// 弾の色の設定
			materail.setDiffuseColor(COLOR_TAMA[time]);
			materail.setSpecularColor(COLOR_TAMA[time]);

		// 跳ね返された場合、
		}else{

			// 位置取得
			double x = super.getTranslateX();
			double y = super.getTranslateY();

			// 移動
			x = x + mx;
			y = y + my;

			// 飛んでったら
			if(x < -20 || x > 20 || y < -10){

				// 削除
				application.removeNode(this);

			}

			// 位置設定
			super.setTranslateX(x);
			super.setTranslateY(y);

		} // end if 跳ね返された場合

	} // end run

	/**
	 * 跳ね返された
	 */
	public void rapel(){

		// 跳ね返された
		flgRapel = true;

		// X移動量
		mx = (Math.random() - 0.5) / 2;

		// Y移動量
		my = -(Math.random() * 0.5) / 2;

		// 跳ね返り音スタート
		application.startWaveRapel();

	} // end rapel

	/**
	 * 跳ね返されているか
	 */
	public boolean isRapel(){
		return flgRapel;
	}

}
