import javafx.scene.Group;
import javafx.scene.shape.MeshView;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.paint.Color;
import javafx.geometry.Point3D;

/**
 * 敵の弾１
 */
public class PRSpriteTamaTeki1 extends PRSpriteTamaTeki0 {

	/** 弾の色 */
	private static final Color[]	COLOR_MISSILE		= new Color[]{
		Color.color(0.8, 0.3, 0.3),
		Color.color(0.8, 0.5, 0.3),
		Color.color(0.8, 0.6, 0.3),
		Color.color(0.8, 0.8, 0.3),
		Color.color(0.8, 0.6, 0.3),
		Color.color(0.8, 0.5, 0.3),
		Color.color(0.8, 0.3, 0.3)
	};

	/** マテリアル */
	private		PhongMaterial		materail		= null;

	/** 時間 */
	private		int					time			= 0;

	/** 速度 */
	private		double				mx				= 0;
	private		double				my				= 0;
	private		double				mz				= 0;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteTamaTeki1.class, "3d/TamaTeki.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ（通常）
	 * <pre>
	 * 通常はこちらのコンストラクタを使用してください。
	 * </pre>
	 * @param application PRApplication
	 * @param sprite 発射するスプライト
	 * @param speed 敵の弾のスピード
	 */
	public PRSpriteTamaTeki1(PRApplication application, Group sprite, double speed) {

		// コンストラクタ呼び出し
		this(application, sprite, null, speed);

	} // end PRSpriteTamaTeki1

	/**
	 * コンストラクタ（発射位置−相対位置）
	 * <pre>
	 * 発射位置を変更したい場合は、こちらのコンストラクタを利用してください。
	 * 相対位置で設定します。
	 * </pre>
	 * @param application PRApplication
	 * @param sprite 発射するスプライト
	 * @param pos 発射相対位置
	 * @param speed 敵の弾のスピード
	 */
	public PRSpriteTamaTeki1(PRApplication application, Group sprite, Point3D pos, double speed) {

		super(application);

		// オブジェクト生成
		ObjReader.create(this);

		// 座標
		double x = sprite.getTranslateX();
		double y = sprite.getTranslateY();
		double z = sprite.getTranslateZ();

		// 発射相対位置が設定されている場合、
		if(pos != null){

			// 座標位置補正
			x = x + pos.getX();
			y = y + pos.getY();
			z = z + pos.getZ();

		}

		// 位置を設定
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// グループ・メッシュビューを取得
		Group group = (Group)this.getChildren().get(0);
		MeshView meshView = (MeshView)(group.getChildren().get(0));

		// マテリアルを取得
		materail = (PhongMaterial)meshView.getMaterial();

		// プレイヤ—を取得
		PRSpritePrusher spritePrusher = application.getPRSpritePrusher();

		// 座標の差を取得
		double saX = x - spritePrusher.getTranslateX();
		double saY = y - spritePrusher.getTranslateY();
		double saZ = z - spritePrusher.getTranslateZ();

		// Ｚ方向の分割数を求める
		double bunkatu = saZ / speed;

		// 速度を設定
		mz = -speed;
		mx = -saX / bunkatu;
		my = -saY / bunkatu;

		// 縮小する
		super.setScaleX(0.5);
		super.setScaleY(0.5);
		super.setScaleZ(0.5);

		// 影を設定
		application.addShadow(this);

	} // end PRSpriteTamaTeki1

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 位置取得
		double x = super.getTranslateX();
		double y = super.getTranslateY();
		double z = super.getTranslateZ();

		// 移動
		x = x + mx;
		y = y + my;
		z = z + mz;

		// 画面の外にでた場合、
		if(z < 0){

			// 削除
			application.removeNode(this);

		}

		// 位置設定
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// 時間設定
		time++;
		time = (time >= 7)?0: time;

		// 弾の色の設定
		materail.setDiffuseColor(COLOR_MISSILE[time]);
		materail.setSpecularColor(COLOR_MISSILE[time]);

		// 回転
		super.setRotate(super.getRotate() + 10);

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 2.0;
	}
}
