import java.util.HashMap;

import javafx.collections.ObservableList;
import javafx.geometry.BoundingBox;
import javafx.geometry.Bounds;
import javafx.geometry.Point3D;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.input.KeyCode;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.MeshView;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Translate;

/**
 * プレイヤー
 */
public class PRSpriteTitlePrusher extends PRSprite {

	/** 右腕 */
	private		Group				groupArmRight		= null;

	/** ファイアー */
	private		Group				groupFire			= null;

	/** 左腕 */
	private		Group				groupArmLeft		= null;

	/** 右足 */
	private		Group				groupLegRight		= null;

	/** 左足 */
	private		Group				groupLegLeft		= null;

	/** 時間 */
	private		int					time				= 0;

	/** 腕の角度 */
	private		int					angleArmLeft		= 0;

	/**
	 * 初期化
	 */
	public static void init() {

		// PRUSHERをそのまま利用するため、ここでは設定しない。

	} // end init

	/**
	 * コンストラクタ
	 */
	public PRSpriteTitlePrusher(PRApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// オブジェクト生成
			HashMap<String, Group> map = ObjReader.create(this, Class.forName("PRSpritePrusher").getName());

			// 各パーツを保持
			groupArmRight = map.get("Arm_Right");
			groupFire = map.get("Fire");
			groupArmLeft = map.get("Arm_Left");
			groupLegRight = map.get("Leg_Right_Leg");
			groupLegLeft = map.get("Leg_Left_Leg");

			// 回転
			groupArmLeft.getTransforms().add(new Rotate(180, Rotate.X_AXIS));

			// 中心点設定
			groupArmRight.getTransforms().add(new Translate(0.0, 0.11, 0.0));
			groupFire.getTransforms().add(new Translate(0.0, 0.10, 0.0));
			groupArmLeft.getTransforms().add(new Translate(0.10, 0.11, 0.0));
			groupLegRight.getTransforms().add(new Translate(0.0, -0.04, 0.04));
			groupLegLeft.getTransforms().add(new Translate(0.0, -0.04, 0.04));

			// 移動させる
			groupArmRight.setTranslateX(groupArmRight.getTranslateX() - 0.042);
			groupFire.setTranslateX(groupFire.getTranslateX() - 0.042);
			groupArmLeft.setTranslateX(groupArmLeft.getTranslateX() - 0.032);
			groupLegRight.setTranslateX(groupLegRight.getTranslateX() - 0.21);
			groupLegLeft.setTranslateX(groupLegLeft.getTranslateX() + 0.21);

			groupArmRight.setTranslateY(groupArmRight.getTranslateY() - 0.112);
			groupFire.setTranslateY(groupFire.getTranslateY() - 0.112);
			groupArmLeft.setTranslateY(groupArmLeft.getTranslateY() - 0.088);
			groupLegRight.setTranslateY(groupLegRight.getTranslateY() + 0.04);
			groupLegLeft.setTranslateY(groupLegLeft.getTranslateY() + 0.04);

			groupLegRight.setTranslateZ(groupLegRight.getTranslateZ() - 0.04);
			groupLegLeft.setTranslateZ(groupLegLeft.getTranslateZ() - 0.04);

			// 回転軸を設定
			super.setRotationAxis(Rotate.Y_AXIS);
			groupArmLeft.setRotationAxis(Rotate.Z_AXIS);
			groupLegRight.setRotationAxis(Rotate.X_AXIS);
			groupLegLeft.setRotationAxis(Rotate.X_AXIS);

			// 回転
			super.setRotate(210);
			groupLegRight.setRotate(60);
			groupLegLeft.setRotate(60);

			// 位置を設定
			this.setTranslateX(0.6);
			this.setTranslateY(-0.9);
			this.setTranslateZ(5.6);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteTitlePrusher

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 時間＋１
		time++;

		// 角度の計算
		time = (time >= 90)?0: time;
		if(time < 45){
			angleArmLeft--;
		}else{
			angleArmLeft++;
		}

		// 腕の角度 
		groupArmLeft.setRotate(angleArmLeft);

	} // end run

}
